@@ -1,31 +1,16 @@
 /*
- * This file is part of the KDE Baloo project.
- * Copyright (C) 2015  Vishesh Handa <vhanda@kde.org>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
- */
+    This file is part of the KDE Baloo project.
+    SPDX-FileCopyrightText: 2015 Vishesh Handa <vhanda@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.1-or-later
+*/
 
 #include "transaction.h"
-#include "postingdb.h"
 #include "documentdb.h"
 #include "documenturldb.h"
 #include "documentiddb.h"
 #include "positiondb.h"
 #include "documentdatadb.h"
-#include "mtimedb.h"
 
 #include "document.h"
 #include "enginequery.h"
@@ -34,7 +19,6 @@
 #include "orpostingiterator.h"
 #include "phraseanditerator.h"
 
-#include "writetransaction.h"
 #include "idutils.h"
 #include "database.h"
 #include "databasesize.h"
@@ -141,13 +125,6 @@ quint64 Transaction::documentId(const QByteArray& path) const
     return parentId;
 }
 
-QVector<quint64> Transaction::childrenDocumentId(quint64 parentId) const
-{
-    DocumentUrlDB docUrlDB(m_dbis.idTreeDbi, m_dbis.idFilenameDbi, m_txn);
-
-    return docUrlDB.getChildren(parentId);
-}
-
 DocumentTimeDB::TimeInfo Transaction::documentTimeInfo(quint64 id) const
 {
     Q_ASSERT(m_txn);
@@ -165,17 +142,6 @@ QByteArray Transaction::documentData(quint64 id) const
     return docDataDb.get(id);
 }
 
-bool Transaction::hasChanges() const
-{
-    Q_ASSERT(m_txn);
-    if (!m_writeTrans) {
-        qCWarning(ENGINE) << "m_writeTrans is null";
-        return false;
-    }
-
-    return m_writeTrans->hasChanges();
-}
-
 QVector<quint64> Transaction::fetchPhaseOneIds(int size) const
 {
     Q_ASSERT(m_txn);
@@ -295,12 +261,12 @@ void Transaction::replaceDocument(const Document& doc, DocumentOperations operat
     m_writeTrans->replaceDocument(doc, operations);
 }
 
-void Transaction::commit()
+bool Transaction::commit()
 {
     Q_ASSERT(m_txn);
     if (!m_writeTrans) {
         qCWarning(ENGINE) << "m_writeTrans is null";
-        return;
+        return false;
     }
 
     m_writeTrans->commit();
@@ -308,11 +274,14 @@ void Transaction::commit()
     m_writeTrans = nullptr;
 
     int rc = mdb_txn_commit(m_txn);
+    m_txn = nullptr;
+
     if (rc) {
         qCWarning(ENGINE) << "Transaction::commit" << mdb_strerror(rc);
+        return false;
     }
 
-    m_txn = nullptr;
+    return true;
 }
 
 void Transaction::abort()
@@ -359,7 +328,7 @@ PostingIterator* Transaction::postingIterator(const EngineQuery& query) const
         vec.reserve(subQueries.size());
         for (const EngineQuery& q : subQueries) {
             if (!q.leaf()) {
-                qCDebug(ENGINE) << "Transaction::toPostingIterator" << "Phrase queries must contain leaf queries";
+                qCDebug(ENGINE) << "Transaction::toPostingIterator" << "Phrase subqueries must be leafs";
                 continue;
             }
             vec << positionDb.iter(q.term());
@@ -401,10 +370,16 @@ PostingIterator* Transaction::postingCompIterator(const QByteArray& prefix, qlon
     return postingDb.compIter(prefix, value, com);
 }
 
-PostingIterator* Transaction::mTimeIter(quint32 mtime, MTimeDB::Comparator com) const
+PostingIterator* Transaction::postingCompIterator(const QByteArray& prefix, double value, PostingDB::Comparator com) const
 {
-    MTimeDB mTimeDb(m_dbis.mtimeDbi, m_txn);
-    return mTimeDb.iter(mtime, com);
+    PostingDB postingDb(m_dbis.postingDbi, m_txn);
+    return postingDb.compIter(prefix, value, com);
+}
+
+PostingIterator* Transaction::postingCompIterator(const QByteArray& prefix, const QByteArray& value, PostingDB::Comparator com) const
+{
+    PostingDB postingDb(m_dbis.postingDbi, m_txn);
+    return postingDb.compIter(prefix, value, com);
 }
 
 PostingIterator* Transaction::mTimeRangeIter(quint32 beginTime, quint32 endTime) const
