@@ -1,21 +1,8 @@
 /*
- * Copyright (C) 2015  Vishesh Handa <vhanda@kde.org>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
- */
+    SPDX-FileCopyrightText: 2015 Vishesh Handa <vhanda@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.1-or-later
+*/
 
 #ifndef BALOO_FILEINDEXSCHEDULER_H
 #define BALOO_FILEINDEXSCHEDULER_H
@@ -43,7 +30,7 @@ class FileIndexScheduler : public QObject
 
     Q_PROPERTY(int state READ state NOTIFY stateChanged)
 public:
-    FileIndexScheduler(Database* db, FileIndexerConfig* config, QObject* parent = nullptr);
+    FileIndexScheduler(Database* db, FileIndexerConfig* config, bool firstRun, QObject* parent = nullptr);
     ~FileIndexScheduler() override;
     int state() const { return m_indexerState; }
 
@@ -54,7 +41,7 @@ public Q_SLOTS:
     void indexNewFile(const QString& file) {
         if (!m_newFiles.contains(file)) {
             m_newFiles << file;
-            if (m_indexerState == Idle || m_isGoingIdle) {
+            if (isIndexerIdle()) {
                 QTimer::singleShot(0, this, &FileIndexScheduler::scheduleIndexing);
             }
         }
@@ -63,7 +50,7 @@ public Q_SLOTS:
     void indexModifiedFile(const QString& file) {
         if (!m_modifiedFiles.contains(file)) {
             m_modifiedFiles << file;
-            if (m_indexerState == Idle || m_isGoingIdle) {
+            if (isIndexerIdle()) {
                 QTimer::singleShot(0, this, &FileIndexScheduler::scheduleIndexing);
             }
         }
@@ -72,7 +59,7 @@ public Q_SLOTS:
     void indexXAttrFile(const QString& file) {
         if (!m_xattrFiles.contains(file)) {
             m_xattrFiles << file;
-            if (m_indexerState == Idle || m_isGoingIdle) {
+            if (isIndexerIdle()) {
                 QTimer::singleShot(0, this, &FileIndexScheduler::scheduleIndexing);
             }
         }
@@ -85,9 +72,11 @@ public Q_SLOTS:
 
     void handleFileRemoved(const QString& file);
 
+    void updateConfig();
     void scheduleIndexing();
     void scheduleCheckUnindexedFiles();
     void scheduleCheckStaleIndexEntries();
+    void startupFinished();
 
     Q_SCRIPTABLE void suspend() { setSuspend(true); }
     Q_SCRIPTABLE void resume() { setSuspend(false); }
@@ -101,6 +90,13 @@ private Q_SLOTS:
 
 private:
     void setSuspend(bool suspend);
+    bool isIndexerIdle() {
+        return m_isGoingIdle ||
+               (m_indexerState == Suspended) ||
+               (m_indexerState == Startup) ||
+               (m_indexerState == Idle) ||
+               (m_indexerState == LowPowerIdle);
+    }
 
     Database* m_db;
     FileIndexerConfig* m_config;
@@ -118,10 +114,15 @@ private:
 
     IndexerState m_indexerState;
     TimeEstimator m_timeEstimator;
+    uint m_indexPendingFiles = 0;
+    uint m_indexFinishedFiles = 0;
 
     bool m_checkUnindexedFiles;
     bool m_checkStaleIndexEntries;
     bool m_isGoingIdle;
+    bool m_isSuspended;
+    bool m_isFirstRun;
+    bool m_inStartup;
 };
 
 }
