@@ -1,49 +1,31 @@
 /*
- * This file is part of the KDE Baloo Project
- * Copyright (C) 2012-2014  Vishesh Handa <me@vhanda.in>
- * Copyright (C) 2017  James D. Smith <smithjd15@gmail.com>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) version 3, or any
- * later version accepted by the membership of KDE e.V. (or its
- * successor approved by the membership of KDE e.V.), which shall
- * act as a proxy defined in Section 6 of version 3 of the license.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
- *
- */
+    This file is part of the KDE Baloo Project
+    SPDX-FileCopyrightText: 2012-2014 Vishesh Handa <me@vhanda.in>
+    SPDX-FileCopyrightText: 2017-2018 James D. Smith <smithjd15@gmail.com>
+    SPDX-FileCopyrightText: 2020 Stefan Brüns <bruns@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
+*/
 
 #include "kio_tags.h"
+#include "kio_tags_debug.h"
 
 #include <QUrl>
-#include <QDebug>
-#include <QLoggingCategory>
 
 #include <KLocalizedString>
 #include <KUser>
-#include <KFileMetaData/UserMetaData>
 #include <KIO/Job>
 
 #include <QCoreApplication>
-#include <QDateTime>
 #include <QDir>
+#include <QRegularExpression>
 
 #include "file.h"
 #include "taglistjob.h"
+#include "../common/udstools.h"
 
-#include "query.h"
 #include "term.h"
 
-Q_LOGGING_CATEGORY(KIO_TAGS, "kf5.kio.kio_tags")
-
 using namespace Baloo;
 
 TagsProtocol::TagsProtocol(const QByteArray& pool_socket, const QByteArray& app_socket)
@@ -85,7 +67,7 @@ void TagsProtocol::stat(const QUrl& url)
             ForwardingSlaveBase::stat(result.fileUrl);
             return;
         case TagUrl:
-            for (const KIO::UDSEntry& entry : result.pathUDSResults) {
+            for (const KIO::UDSEntry& entry : qAsConst(result.pathUDSResults)) {
                 if (entry.stringValue(KIO::UDSEntry::UDS_EXTRA) == result.tag) {
                     statEntry(entry);
                 }
@@ -191,7 +173,8 @@ void TagsProtocol::rename(const QUrl& src, const QUrl& dest, KIO::JobFlags flags
             if (it.filePath() == srcResult.fileUrl.toLocalFile()) {
                 rewriteTags(md, srcResult.tag, dstResult.tag);
             } else if (srcResult.fileUrl.isEmpty()) {
-                for (const QString& tag : md.tags()) {
+                const auto tags = md.tags();
+                for (const QString& tag : tags) {
                     if (tag == srcResult.tag || (tag.startsWith(srcResult.tag + QLatin1Char('/')))) {
                         QString newTag = tag;
                         newTag.replace(srcResult.tag, dstResult.tag, Qt::CaseInsensitive);
@@ -231,8 +214,9 @@ void TagsProtocol::del(const QUrl& url, bool isfile)
                 if (it.filePath() == result.fileUrl.toLocalFile()) {
                     rewriteTags(md, result.tag);
                 } else if (result.fileUrl.isEmpty()) {
-                    for (const QString &tag : md.tags()) {
-                        if ((tag == result.tag) || (tag.startsWith(QString(result.tag + QLatin1Char('/')), Qt::CaseInsensitive))) {
+                    const auto tags = md.tags();
+                    for (const QString &tag : tags) {
+                        if ((tag == result.tag) || (tag.startsWith(result.tag + QLatin1Char('/'), Qt::CaseInsensitive))) {
                             rewriteTags(md, tag);
                         }
                     }
@@ -294,15 +278,21 @@ TagsProtocol::ParseResult TagsProtocol::parseUrl(const QUrl& url, const QList<Pa
     TagsProtocol::ParseResult result;
     result.decodedUrl = QUrl::fromPercentEncoding(url.toString().toUtf8());
 
+    if ((url.scheme() == QLatin1String("tags")) && result.decodedUrl.length()>6 && result.decodedUrl.at(6) == QChar('/')) {
+        result.urlType = InvalidUrl;
+        return result;
+    }
+
     auto createUDSEntryForTag = [] (const QString& tagSection, const QString& tag) {
         KIO::UDSEntry uds;
-        uds.insert(KIO::UDSEntry::UDS_NAME, tagSection);
-        uds.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
-        uds.insert(KIO::UDSEntry::UDS_MIME_TYPE, QStringLiteral("inode/directory"));
-        uds.insert(KIO::UDSEntry::UDS_ACCESS, 0700);
-        uds.insert(KIO::UDSEntry::UDS_USER, KUser().loginName());
-        uds.insert(KIO::UDSEntry::UDS_ICON_NAME, QStringLiteral("tag"));
-        uds.insert(KIO::UDSEntry::UDS_EXTRA, tag);
+        uds.reserve(9);
+        uds.fastInsert(KIO::UDSEntry::UDS_NAME, tagSection);
+        uds.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+        uds.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, QStringLiteral("inode/directory"));
+        uds.fastInsert(KIO::UDSEntry::UDS_ACCESS, 0700);
+        uds.fastInsert(KIO::UDSEntry::UDS_USER, KUser().loginName());
+        uds.fastInsert(KIO::UDSEntry::UDS_ICON_NAME, QStringLiteral("tag"));
+        uds.fastInsert(KIO::UDSEntry::UDS_EXTRA, tag);
 
         QString displayType;
         if (tagSection == tag) {
@@ -313,16 +303,16 @@ TagsProtocol::ParseResult TagsProtocol::parseUrl(const QUrl& url, const QList<Pa
             displayType = i18n("All Tags");
         }
 
-        uds.insert(KIO::UDSEntry::UDS_DISPLAY_TYPE, displayType);
+        uds.fastInsert(KIO::UDSEntry::UDS_DISPLAY_TYPE, displayType);
 
         QString displayName = i18n("All Tags");
-        if (!tag.isEmpty() && ((tagSection == QStringLiteral(".")) || (tagSection == QStringLiteral("..")))) {
+        if (!tag.isEmpty() && ((tagSection == QLatin1Char('.')) || (tagSection == QLatin1String("..")))) {
             displayName = tag.section(QLatin1Char('/'), -1);
         } else {
             displayName = tagSection;
         }
 
-        uds.insert(KIO::UDSEntry::UDS_DISPLAY_NAME, displayName);
+        uds.fastInsert(KIO::UDSEntry::UDS_DISPLAY_NAME, displayName);
 
         return uds;
     };
@@ -337,28 +327,48 @@ TagsProtocol::ParseResult TagsProtocol::parseUrl(const QUrl& url, const QList<Pa
         result.urlType = FileUrl;
         result.fileUrl = url;
         result.metaData = KFileMetaData::UserMetaData(url.toLocalFile());
-        qCDebug(KIO_TAGS) << result.decodedUrl << "url file path:" << result.fileUrl.toLocalFile();
     } else if (url.scheme() == QLatin1String("tags")) {
         bool validTag = flags.contains(LazyValidation);
-        QString filePath;
-
-        // Extract any local file path from the URL.
-        if (result.decodedUrl.contains(QStringLiteral("?/"))) {
-            filePath = QDir::separator() + result.decodedUrl.section(QStringLiteral("?/"), -1, QString::SectionSkipEmpty);
-            result.fileUrl = QUrl::fromLocalFile(filePath);
-            result.metaData = KFileMetaData::UserMetaData(filePath);
-            qCDebug(KIO_TAGS) << result.decodedUrl << "url file path:" << filePath;
-        }
 
         // Determine the tag from the URL.
-        result.tag = result.decodedUrl.section(QStringLiteral("?/"), 0, 0);
+        result.tag = result.decodedUrl;
         result.tag.remove(url.scheme() + QLatin1Char(':'));
         result.tag = QDir::cleanPath(result.tag);
         while (result.tag.startsWith(QLatin1Char('/'))) {
             result.tag.remove(0, 1);
         }
 
-        if (!filePath.isEmpty() || flags.contains(ChopLastSection)) {
+        // Extract any local file path from the URL.
+        QString tag = result.tag.section(QDir::separator(), 0, -2);
+        QString fileName = result.tag.section(QDir::separator(), -1, -1);
+        int pos = 0;
+
+        // Extract and remove any multiple filename suffix from the file name.
+        QRegularExpression regexp(QStringLiteral("\\s\\((\\d+)\\)$"));
+        QRegularExpressionMatch regMatch = regexp.match(fileName);
+        if (regMatch.hasMatch()) {
+            pos = regMatch.captured(1).toInt();
+
+            fileName.remove(regexp);
+        }
+
+        Query q;
+        q.setSearchString(QStringLiteral("tag=\"%1\" AND filename=\"%2\"").arg(tag, fileName));
+        ResultIterator it = q.exec();
+
+        int i = 0;
+        while (it.next()) {
+            result.fileUrl = QUrl::fromLocalFile(it.filePath());
+            result.metaData = KFileMetaData::UserMetaData(it.filePath());
+
+            if (i == pos) {
+                break;
+            } else {
+                i++;
+            }
+        }
+
+        if (!result.fileUrl.isEmpty() || flags.contains(ChopLastSection)) {
             result.tag = result.tag.section(QDir::separator(), 0, -2);
         }
 
@@ -373,14 +383,6 @@ TagsProtocol::ParseResult TagsProtocol::parseUrl(const QUrl& url, const QList<Pa
             result.query.setSearchString(query);
 
             qCDebug(KIO_TAGS) << result.decodedUrl << "url query:" << query;
-
-            if (result.tag.contains(QLatin1Char('/'))) {
-                qCDebug(KIO_TAGS) << result.decodedUrl << "url is a tag fragment:" << result.tag;
-            } else {
-                qCDebug(KIO_TAGS) << result.decodedUrl << "url is a tag:" << result.tag;
-            }
-        } else {
-            qCDebug(KIO_TAGS) << result.decodedUrl << "url is the root tag url";
         }
 
         // Create the tag directory entries.
@@ -394,8 +396,6 @@ TagsProtocol::ParseResult TagsProtocol::parseUrl(const QUrl& url, const QList<Pa
                 if (!tagPaths.contains(tagSection, Qt::CaseInsensitive) && !tagSection.isEmpty()) {
                     result.pathUDSResults << createUDSEntryForTag(tagSection, tag);
                     tagPaths << tagSection;
-
-                    qCDebug(KIO_TAGS) << result.decodedUrl << "added path:" << tagSection;
                 }
             }
 
@@ -409,15 +409,13 @@ TagsProtocol::ParseResult TagsProtocol::parseUrl(const QUrl& url, const QList<Pa
         }
     }
 
-    qCDebug(KIO_TAGS) << result.decodedUrl << "url type:" << result.urlType;
-
     if (result.urlType == FileUrl) {
         return result;
     } else {
         result.pathUDSResults << createUDSEntryForTag(QStringLiteral("."), result.tag);
     }
 
-    // The root tag url has no file entries
+    // The root tag url has no file entries.
     if (result.tag.isEmpty()) {
         return result;
     } else {
@@ -426,31 +424,26 @@ TagsProtocol::ParseResult TagsProtocol::parseUrl(const QUrl& url, const QList<Pa
 
     // Query for any files associated with the tag.
     Query q;
-    q.setSortingOption(Query::SortNone);
     q.setSearchString(QStringLiteral("tag=\"%1\"").arg(result.tag));
     ResultIterator it = q.exec();
+    QList<QString> resultNames;
+    UdsFactory udsf;
+
     while (it.next()) {
-        const QUrl& match = QUrl::fromLocalFile(it.filePath());
+        KIO::UDSEntry uds = udsf.createUdsEntry(it.filePath());
+        if (uds.count() == 0) {
+	    continue;
+	}
 
-        // Somehow stat the file
-        KIO::UDSEntry uds;
-        KIO::StatJob* job = KIO::stat(match, KIO::HideProgressInfo);
-        // we do not want to wait for the event loop to delete the job
-        QScopedPointer<KIO::StatJob> sp(job);
-        job->setAutoDelete(false);
-        if (job->exec()) {
-            uds = job->statResult();
-
-            qCDebug(KIO_TAGS) << result.decodedUrl << "adding file:" << match.fileName();
-        } else {
-            continue;
+	const QUrl url(uds.stringValue(KIO::UDSEntry::UDS_URL));
+	auto dupCount = resultNames.count(url.fileName());
+        if (dupCount > 0) {
+            uds.replace(KIO::UDSEntry::UDS_NAME, url.fileName() + QStringLiteral(" (%1)").arg(dupCount));
         }
 
-        uds.insert(KIO::UDSEntry::UDS_NAME, match.fileName() + QStringLiteral("?") + it.filePath());
-        uds.insert(KIO::UDSEntry::UDS_DISPLAY_NAME, match.fileName());
-        uds.insert(KIO::UDSEntry::UDS_TARGET_URL, match.toString());
-        uds.insert(KIO::UDSEntry::UDS_ICON_OVERLAY_NAMES, QStringLiteral("tag"));
+        qCDebug(KIO_TAGS) << result.tag << "adding file:" << uds.stringValue(KIO::UDSEntry::UDS_NAME);
 
+        resultNames << url.fileName();
         result.pathUDSResults << uds;
     }
 
